//
//  MTTexturePlugIn.m
//  MTTexture
//
//  Created by Daniel Morena on 2/15/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "MTTexturePlugIn.h"

#define	kQCPlugIn_Name				@"MTTexture"
#define	kQCPlugIn_Description		@"MTTexture description"

@implementation MTTexturePlugIn

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/
@synthesize groupImport;

@dynamic inputFetchSignal;


+ (NSDictionary*) attributes
{
	/*
	Return a dictionary of attributes describing the plug-in (QCPlugInAttributeNameKey, QCPlugInAttributeDescriptionKey...).
	*/
	
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
	/*
	Specify the optional attributes for property based ports (QCPortAttributeNameKey, QCPortAttributeDefaultValueKey...).
	*/
	
    if([key isEqualToString:@"inputFetchSignal"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Fecth Signal",QCPortAttributeNameKey, nil];
    }
    
	return nil;
}

+ (QCPlugInExecutionMode) executionMode
{
	return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
	return kQCPlugInTimeModeNone;
}

+(NSArray*)plugInKeys
{
	return [NSArray arrayWithObjects:@"groupImport",nil];
}

-(void)setSerializedValue:(id)serializedValue forKey:(NSString*)key
{
    [super setSerializedValue:serializedValue forKey:key];
    
    if([key isEqualToString:@"groupImport"])
    {
        [self managePortsForGroupImportState];
    }
}

-(void)managePortsForGroupImportState
{
    if(groupImport)
    {
        for(NSDictionary *singlePort in singlePorts)
        {
            if([currentPorts containsObject:[singlePort objectForKey:@"key"]])
            {
                if([[singlePort objectForKey:@"input"] boolValue])
                {
                    [self removeInputPortForKey:[singlePort objectForKey:@"key"]];
                }
                else
                {
                    [self removeOutputPortForKey:[singlePort objectForKey:@"key"]];
                }
                
                [currentPorts removeObject:[singlePort objectForKey:@"key"]];
            }
        }
        
        for(NSDictionary *groupPort in groupPorts)
        {
            if([currentPorts containsObject:[groupPort objectForKey:@"key"]])
            {
                continue;
            }
            
            if([[groupPort objectForKey:@"input"] boolValue])
            {
                [self addInputPortWithType:[groupPort objectForKey:@"type"] 
                                    forKey:[groupPort objectForKey:@"key"]
                            withAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[groupPort objectForKey:@"name"],QCPortAttributeNameKey, nil]];
            }
            else
            {
                [self addOutputPortWithType:[groupPort objectForKey:@"type"] 
                                    forKey:[groupPort objectForKey:@"key"] 
                            withAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[groupPort objectForKey:@"name"],QCPortAttributeNameKey, nil]];
            }
            
            [currentPorts addObject:[groupPort objectForKey:@"key"]];
        }
        
    }
    else
    {
        for(NSDictionary *groupPort in groupPorts)
        {
            if([currentPorts containsObject:[groupPort objectForKey:@"key"]])
            {
                if([[groupPort objectForKey:@"input"] boolValue])
                {
                    [self removeInputPortForKey:[groupPort objectForKey:@"key"]];
                }
                else
                {
                    [self removeOutputPortForKey:[groupPort objectForKey:@"key"]];
                }
                [currentPorts removeObject:[groupPort objectForKey:@"key"]];
            }
        }
        
        [currentPorts removeObjectsInArray:groupPorts];
        
        for(NSDictionary *singlePort in singlePorts)
        {
            if([currentPorts containsObject:[singlePort objectForKey:@"key"]])
            {
                continue;
            }

            if([[singlePort objectForKey:@"input"] boolValue])
            {
                [self addInputPortWithType:[singlePort objectForKey:@"type"] 
                                    forKey:[singlePort objectForKey:@"key"]
                            withAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[singlePort objectForKey:@"name"],QCPortAttributeNameKey, nil]];
            }
            else
            {
                [self addOutputPortWithType:[singlePort objectForKey:@"type"] 
                                    forKey:[singlePort objectForKey:@"key"] 
                            withAttributes:[NSDictionary dictionaryWithObjectsAndKeys:[singlePort objectForKey:@"name"],QCPortAttributeNameKey, nil]];
            }
            
            [currentPorts addObject:[singlePort objectForKey:@"key"]];
        }
    }
}

-(void)setGroupImport:(BOOL)value
{
    groupImport=value;
    [self managePortsForGroupImportState];
}

-(QCPlugInViewController*)createViewController
{
	return [[QCPlugInViewController alloc] initWithPlugIn:self viewNibName:@"Settings"];
}

- (id) init
{
	if(self = [super init]) {

        currentPorts=[NSMutableArray new];
        groupPorts=[[NSArray alloc] initWithObjects:
                    [NSDictionary dictionaryWithObjectsAndKeys:@"inputImagesPath",@"key",QCPortTypeString,@"type",[NSNumber numberWithBool:YES],@"input",@"Images Path",@"name", nil]
                    ,[NSDictionary dictionaryWithObjectsAndKeys:@"outputTextures",@"key",[NSNumber numberWithBool:NO],@"input",QCPortTypeStructure,@"type",@"Textures Names",@"name", nil]
                    , nil];
        singlePorts=[[NSArray alloc] initWithObjects:
                    [NSDictionary dictionaryWithObjectsAndKeys:@"inputImagePath",@"key",QCPortTypeString,@"type",[NSNumber numberWithBool:YES],@"input",@"Image Path",@"name", nil]
                     ,[NSDictionary dictionaryWithObjectsAndKeys:@"outputTextureName",@"key",[NSNumber numberWithBool:NO],@"input",QCPortTypeNumber,@"type",@"Texture Name",@"name", nil]
                     ,[NSDictionary dictionaryWithObjectsAndKeys:@"outputTextureWidth",@"key",[NSNumber numberWithBool:NO],@"input",QCPortTypeNumber,@"type",@"Texture Width",@"name", nil]
                     ,[NSDictionary dictionaryWithObjectsAndKeys:@"outputTextureHeight",@"key",[NSNumber numberWithBool:NO],@"input",QCPortTypeNumber,@"type",@"Texture Height",@"name", nil]
                    , nil];
	}
	
	return self;
}

- (void) dealloc
{
	[currentPorts release];
    [groupPorts release];
    [singlePorts release];
	[super dealloc];
}


-(void)manageCurrentTexturesWithContext:(id<QCPlugInContext>)context
{
    [self releaseTexturesWithContext:context];
    textures=[NSMutableArray new];
}

-(void)releaseTexturesWithContext:(id<QCPlugInContext>)context
{
    CGLContextObj cgl_ctx=[context CGLContextObj];
    
    if(nil!=textures)
    {
        for(NSDictionary *texture in textures)
        {
            GLuint name=(GLuint)[[texture valueForKey:@"textureName"] intValue];
            glDeleteTextures(1, &name);
        }
        
        [textures removeAllObjects];
        [textures release];
        
        textures=nil;
    }
}

-(NSDictionary*)allocTextureWithFilePath:(NSString*)path andContext:(id<QCPlugInContext>)context
{
    CGLContextObj cgl_ctx=[context CGLContextObj];
    
    //grad the file definitions
    CFStringRef fileExtension = (CFStringRef) [path pathExtension];
    CFStringRef fileUTI = UTTypeCreatePreferredIdentifierForTag(kUTTagClassFilenameExtension, fileExtension, NULL);
    
    //the pointer to the texture name
    GLuint texName;
    
    //the size
    GLuint texWidth=0;
    GLuint texHeight=0;
    
    //if file is a valid image...
    if (UTTypeConformsTo(fileUTI, kUTTypeImage))
    {
        //the image to be loaded
        NSData *image=[[NSData alloc] initWithContentsOfURL:[NSURL fileURLWithPath:path]];
        //grab a image representation
        NSBitmapImageRep *bitmap = [NSBitmapImageRep imageRepWithData:image];
        //tell opengl to create one texture and pass it over to texture name
        glGenTextures(1,&texName);
        //bind the texture, this will it activate it
        glBindTexture(GL_TEXTURE_2D,texName);
        //cache the textures size
        texWidth=(GLuint)[bitmap size].width;
        texHeight=(GLuint)[bitmap size].height;
        // Set proper unpacking row length for bitmap.
        glPixelStorei(GL_UNPACK_ROW_LENGTH, [bitmap pixelsWide]);
        // Set byte aligned unpacking (needed for 3 byte per pixel bitmaps).
        glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_CLAMP);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
        glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
        //generate the texture
        glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, texWidth, texHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, [bitmap bitmapData]);
        //release the temp image...
        [image release];
    }
    //release the file
    CFRelease(fileUTI);
    //the entry for texture tracking
    return [NSDictionary dictionaryWithObjectsAndKeys:[NSNumber numberWithInt:(int)texName],@"textureName",[NSNumber numberWithInt:(int)texWidth],@"textureWidth",[NSNumber numberWithInt:(int)texHeight],@"textureHeight", nil ];
}


@end

@implementation MTTexturePlugIn (Execution)

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
    CGLContextObj cgl_ctx=[context CGLContextObj];
	
	if(cgl_ctx==NULL) 
	{
		return NO; //no context, no gain :)
	}

    if(groupImport)
    {
        if([self didValueForInputKeyChange:@"inputImagesPath"] && [self valueForInputKey:@"inputImagesPath"]!=nil)
        {
            [self manageCurrentTexturesWithContext:context];
            
            NSString *inputPath=[self valueForInputKey:@"inputImagesPath"];
            NSString *compositionPath=[[context compositionURL] path];
            NSString *path=nil;
            
            if([inputPath isAbsolutePath])
            {
                path=inputPath;
            }
            else
            {
                path=[NSString pathWithComponents:
                      [NSArray arrayWithObjects:[compositionPath stringByDeletingLastPathComponent]
                       , inputPath
                       , nil]];
            }
            
            BOOL isDir;
            
            NSFileManager *fm=[NSFileManager defaultManager];
            
            if([fm fileExistsAtPath:path isDirectory:&isDir])
            {
                
                if(isDir)
                {
                    NSError *error=nil;
                    NSArray *files=[fm contentsOfDirectoryAtURL:[NSURL fileURLWithPath:path] 
                                     includingPropertiesForKeys:nil 
                                                        options:NSDirectoryEnumerationSkipsSubdirectoryDescendants
                                    |NSDirectoryEnumerationSkipsPackageDescendants
                                    |NSDirectoryEnumerationSkipsHiddenFiles
                                                          error:&error];
                    
                    unsigned count=[files count];
                    
                    if(count==0) return YES;
                    
                    for(unsigned i=0; i < count; i++)
                    {
                        BOOL isDir;
                        
                        NSURL *file=[files objectAtIndex:i];
                        
                        if([fm fileExistsAtPath:[file path] isDirectory:&isDir])
                        {
                            if(!isDir)
                            {
                                [textures addObject:[self allocTextureWithFilePath:[file path] andContext:context]];
                            }
                        }
                    }
                }
            }
            
            if(textures && [textures count] > 0)
            {
                [self setValue:[NSDictionary dictionaryWithObject:textures forKey:@"textures"] forOutputKey:@"outputTextures"];
            }
        }
    }
    else
    {
        if([self didValueForInputKeyChange:@"inputImagePath"] && [self valueForInputKey:@"inputImagePath"]!=nil)
        {
            NSString *inputPath=[self valueForInputKey:@"inputImagePath"];
            NSString *compositionPath=[[context compositionURL] path];
            NSString *path=nil;
            
            if([inputPath isAbsolutePath])
            {
                path=inputPath;
            }
            else
            {
                path=[NSString pathWithComponents:
                      [NSArray arrayWithObjects:[compositionPath stringByDeletingLastPathComponent]
                       , inputPath
                       , nil]];
            }
            
            BOOL isDir;
            
            NSFileManager *fm=[NSFileManager defaultManager];
            
            if([fm fileExistsAtPath:path isDirectory:&isDir])
            {
                if(!isDir)
                {
                    [self manageCurrentTexturesWithContext:context];
                    [textures addObject:[self allocTextureWithFilePath:[self valueForInputKey:@"inputImagePath"] andContext:context]];
                }
            }
        }
        
        if(textures && [textures count] > 0)
        {
            NSDictionary *entry=[textures objectAtIndex:0];

            [self setValue:[entry objectForKey:@"textureName"] forOutputKey:@"outputTextureName"];
            [self setValue:[entry objectForKey:@"textureWidth"] forOutputKey:@"outputTextureWidth"];
            [self setValue:[entry objectForKey:@"textureHeight"] forOutputKey:@"outputTextureHeight"];
        }
    }
    
	return YES;
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
    [self releaseTexturesWithContext:context];
}

@end
