//
//  MTTexturePlugIn.m
//  MTTexture
//
//  Created by Daniel Morena on 2/15/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

/* It's highly recommended to use CGL macros instead of changing the current context for plug-ins that perform OpenGL rendering */
#import <OpenGL/CGLMacro.h>

#import "MTTextureSpritePlugIn.h"

#define	kQCPlugIn_Name				@"MTTextureSprite"
#define	kQCPlugIn_Description		@"MTTextureSprite description"

@implementation MTTextureSpritePlugIn

/*
Here you need to declare the input / output properties as dynamic as Quartz Composer will handle their implementation
@dynamic inputFoo, outputBar;
*/

@dynamic
inputTextureName
, inputPositionX
, inputPositionY
, inputPositionZ
, inputRotationX
, inputRotationY
, inputRotationZ
, inputWidth
, inputHeight
, inputColorRed
, inputColorGreen
, inputColorBlue
, inputColorAlpha
, inputBlending
, inputDepthTesting
;

+ (NSDictionary*) attributes
{
	return [NSDictionary dictionaryWithObjectsAndKeys:kQCPlugIn_Name
			, QCPlugInAttributeNameKey, kQCPlugIn_Description, QCPlugInAttributeDescriptionKey, nil];
}

+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
    if([key isEqualToString:@"inputTextureName"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Texture Name",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputPositionX"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"X Position",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputPositionY"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Y Position",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputPositionZ"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Z Position",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputRotationX"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"X Rotation",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputRotationY"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Y Rotation",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputRotationZ"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Z Rotation",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputWidth"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Width",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputHeight"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Height",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"inputColorRed"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Color Red",QCPortAttributeNameKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeMaximumValueKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeDefaultValueKey
                ,[NSNumber numberWithDouble:0.0], QCPortAttributeMinimumValueKey
                , nil];
    }
    if([key isEqualToString:@"inputColorGreen"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Color Green",QCPortAttributeNameKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeMaximumValueKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeDefaultValueKey
                ,[NSNumber numberWithDouble:0.0], QCPortAttributeMinimumValueKey
                , nil];
    }
    if([key isEqualToString:@"inputColorBlue"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Color Blue",QCPortAttributeNameKey                
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeMaximumValueKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeDefaultValueKey
                ,[NSNumber numberWithDouble:0.0], QCPortAttributeMinimumValueKey
                , nil];

    }
    if([key isEqualToString:@"inputColorAlpha"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Color Alpha",QCPortAttributeNameKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeMaximumValueKey
                ,[NSNumber numberWithDouble:1.0], QCPortAttributeDefaultValueKey
                ,[NSNumber numberWithDouble:0.0], QCPortAttributeMinimumValueKey
                , nil];

    }
    if([key isEqualToString:@"inputDepthTesting"])
    {
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Depth Testing", QCPortAttributeNameKey,
                [NSArray arrayWithObjects:@"None", @"Read / Write", @"Read Only", nil], QCPortAttributeMenuItemsKey,
                [NSNumber numberWithUnsignedInt:2], QCPortAttributeMaximumValueKey,
                [NSNumber numberWithUnsignedInt:1], QCPortAttributeDefaultValueKey,
                [NSNumber numberWithUnsignedInt:0], QCPortAttributeMinimumValueKey, nil];
    }
    if([key isEqualToString:@"inputBlending"])
    {
		return [NSDictionary dictionaryWithObjectsAndKeys:@"Blending", QCPortAttributeNameKey,
                [NSArray arrayWithObjects:@"Replace", @"Over", @"Add", @"Alpha", nil], QCPortAttributeMenuItemsKey,
                [NSNumber numberWithUnsignedInt:3], QCPortAttributeMaximumValueKey,
                [NSNumber numberWithUnsignedInt:0], QCPortAttributeDefaultValueKey,
                [NSNumber numberWithUnsignedInt:0], QCPortAttributeMinimumValueKey, nil];
    }
	return nil;
}

+(NSArray*)sortedPropertyPortKeys
{
    return [NSArray arrayWithObjects:
            @"inputTextureName"
            ,@"inputPositionX"
            ,@"inputPositionY"
            ,@"inputPositionZ"
            ,@"inputRotationX"
            ,@"inputRotationY"
            ,@"inputRotationZ"
            ,@"inputWidth"
            ,@"inputHeight"
            ,@"inputColorRed"
            ,@"inputColorGreen"
            ,@"inputColorBlue"
            ,@"inputColorAlpha"
            ,@"inputBlending"
            ,@"inputDepthTesting"
            ,nil];
}
+ (QCPlugInExecutionMode) executionMode
{
	return kQCPlugInExecutionModeConsumer;
}

+ (QCPlugInTimeMode) timeMode
{
	return kQCPlugInTimeModeNone;
}

@end

@implementation MTTextureSpritePlugIn(Execution)

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
    CGLContextObj cgl_ctx=[context CGLContextObj];
	
    //*/

	if(cgl_ctx==NULL) 
	{
		return NO; //no context, no gain :)
	}

    //state save
    GLint saveMode;
    glGetIntegerv(GL_MATRIX_MODE, &saveMode);
    glMatrixMode( GL_MODELVIEW );
    glPushMatrix();
    
    //glClear(GL_DEPTH_BUFFER_BIT);
    
    
    if(self.inputBlending == 0)
    {
        glDisable(GL_BLEND);
    }
    else if(self.inputBlending == 1)
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_ONE, GL_ONE_MINUS_SRC_ALPHA);
    }
    else if(self.inputBlending == 2)
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_ONE, GL_ONE);
    }
    else if(self.inputBlending == 3)
    {
        glEnable(GL_BLEND);
        glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
    }
    
    if(self.inputDepthTesting==0)
    {
        glDisable(GL_DEPTH_TEST);
    }
    else if(self.inputDepthTesting==1)
    {
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LESS);
        glDepthMask(GL_TRUE);
    }
    else if(self.inputDepthTesting==2)
    {
        glEnable(GL_DEPTH_TEST);
        glDepthFunc(GL_LEQUAL);
        //glDepthMask(GL_TRUE);
    }

    //move the texture
    glTranslatef(self.inputPositionX, self.inputPositionY, self.inputPositionZ);
    
    //rotate it.
    glRotatef(self.inputRotationX, 1.0, 0.0, 0.0);
    glRotatef(self.inputRotationY, 0.0, 1.0, 0.0);
    glRotatef(self.inputRotationZ, 0.0, 0.0, 1.0);
    
    //just print the image
    glEnable(GL_TEXTURE_2D);
    glBindTexture(GL_TEXTURE_2D, (GLuint)self.inputTextureName);

    glColor4f(self.inputColorRed, self.inputColorGreen, self.inputColorBlue, self.inputColorAlpha);
    
    float qcTexWidth=self.inputWidth;
    float qcTexHeight=self.inputHeight;
    float qcTexX=-qcTexWidth/2;
    float qcTexY=qcTexHeight/2;
        
    glBegin(GL_QUADS);
    //{
        glTexCoord2f(1.0, 1.0); glVertex3f(qcTexX + qcTexWidth, qcTexY - qcTexHeight, 0);	// upper right 
        glTexCoord2f(0.0, 1.0); glVertex3f(qcTexX, qcTexY - qcTexHeight, 0);				// upper left
        glTexCoord2f(0.0, 0.0); glVertex3f(qcTexX, qcTexY, 0);								// lower left
        glTexCoord2f(1.0, 0.0); glVertex3f(qcTexX + qcTexWidth, qcTexY, 0);					// lower right
    ///}
    glEnd();
    
    //state restore
    glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
	glMatrixMode(saveMode);
    
    glBindTexture(GL_TEXTURE_2D,0);
    glDisable(GL_TEXTURE_2D);
    glDisable(GL_DEPTH_TEST);
    glDisable(GL_BLEND);
    
	return YES;
}

@end
