//
//  MTTextureTypePlugIn.m
//  MTTexture
//
//  Created by Daniel Morena on 3/3/12.
//  Copyright (c) 2012 __MyCompanyName__. All rights reserved.
//

#import <OpenGL/CGLMacro.h>

#import "MTTextureTypePlugIn.h"

@implementation MTTextureTypePlugIn

@dynamic 
 inputPurge
//, inputCSS
//, inputHTML
, inputText
, inputFontName
, inputFontSize
, inputFontLeading
, inputFontKerning
, inputWidth
, inputHeight
, outputTextureName
, outputTextureWidth
, outputTextureHeight;

+ (NSDictionary*) attributes
{
    return [NSDictionary dictionaryWithObjectsAndKeys:@"MTTextureType",QCPlugInAttributeNameKey
            ,@"MTTextureType Description",QCPlugInAttributeDescriptionKey, nil];
}

/*
 Override this method to specify the optional attributes for property based ports (QCPortAttribute...).
 */
+ (NSDictionary*) attributesForPropertyPortWithKey:(NSString*)key
{
    if([key isEqualToString:@"inputPurge"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Purge Textures",QCPortAttributeNameKey,[NSNumber numberWithBool:YES], QCPortAttributeDefaultValueKey, nil];
    }
     if([key isEqualToString:@"inputText"])
     {
     return [NSDictionary dictionaryWithObjectsAndKeys:@"Text",QCPortAttributeNameKey, @"Hello.", QCPortAttributeDefaultValueKey, nil];
     }
     
    if([key isEqualToString:@"inputFontName"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Font Name",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"inputFontSize"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Font Size",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"inputFontLeading"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Font Leading",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"inputFontKerning"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Font Kerning",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"inputWidth"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Texture Target Width",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"inputHeight"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Texture Target Height",QCPortAttributeNameKey, nil];
    }
    if([key isEqualToString:@"outputTextureName"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Texture Name",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"outputTextureWidth"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Texture Width",QCPortAttributeNameKey, nil];
    }
    
    if([key isEqualToString:@"outputTextureHeight"])
    {
        return [NSDictionary dictionaryWithObjectsAndKeys:@"Texture Height",QCPortAttributeNameKey, nil];
    }
    
    return nil;
}

+ (NSArray*) sortedPropertyPortKeys
{
    return [NSArray arrayWithObjects:@"inputHTML",@"inputCSS",@"inputPurge",@"outputTextureName",@"outputTextureWidth",@"outputTextureHeight",nil];
}

+ (QCPlugInExecutionMode) executionMode
{
    return kQCPlugInExecutionModeProcessor;
}

+ (QCPlugInTimeMode) timeMode
{
    return kQCPlugInTimeModeNone;
}

/*
+ (NSArray*) plugInKeys
{
    return nil;
}

- (id) serializedValueForKey:(NSString*)key
{
    return [super serializedValueForKey:key];
}

- (void) setSerializedValue:(id)serializedValue forKey:(NSString*)key
{
    [super setSerializedValue:serializedValue forKey:key];
}

-(QCPlugInViewController*)createViewController
{
	return [[QCPlugInViewController alloc] initWithPlugIn:self viewNibName:@"Settings-Type"];
}
 */

-(id)init
{
    if(self=[super init])
    {
        textures=[NSMutableArray new];
    }
    
    return self;
}

-(void)dealloc
{
    [super dealloc];
}

-(NSDictionary*)allocTextureWithString:(NSString*)text 
                   attributes:(NSDictionary*)attributes
                   andContext:(id<QCPlugInContext>)context
{
    //create and upload the texture;
    CGLContextObj cgl_ctx = [context CGLContextObj];
    //findout the size of the element, letter or word.
    NSSize size = [text sizeWithAttributes:attributes];
    //create the temp image, which will hold the font offscreen render
    NSImage *image = [[NSImage alloc] initWithSize:size];
    //lock the view, so any draw context will be registered in the image
    [image lockFocus];
    //call the draw command
        //maybe use drawInRect
    [text drawAtPoint:NSMakePoint(0,0) withAttributes:attributes];
    //and then release the command
    [image unlockFocus];
    //grad the data into a bitmap form.
    NSBitmapImageRep *bitmap = [NSBitmapImageRep imageRepWithData:[image TIFFRepresentation]];
    //enable texture mode
    glEnable(GL_TEXTURE_2D);
    //create the name for texture
    GLuint textureName;
    //tell opengl to create one texture spot and pass it over to texture name pointer
    glGenTextures(1,&textureName);
    //bind the texture, this will it activate it
    glBindTexture(GL_TEXTURE_2D,textureName);
    //assign the texture properties
    // Set proper unpacking row length for bitmap.
    glPixelStorei(GL_UNPACK_ROW_LENGTH, [bitmap pixelsWide]);
    // Set byte aligned unpacking (needed for 3 byte per pixel bitmaps).
    glPixelStorei (GL_UNPACK_ALIGNMENT, 1);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_S, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_WRAP_T, GL_CLAMP);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
    glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
    
    //register the texture size
    GLuint textureWidth=(GLuint)[bitmap size].width;
    GLuint textureHeight=(GLuint)[bitmap size].height;
    
    //... and finally generate the texture
    glTexImage2D(GL_TEXTURE_2D, 0, GL_RGBA, textureWidth, textureHeight, 0, GL_RGBA, GL_UNSIGNED_BYTE, [bitmap bitmapData]);
    //unset the state machine
    glBindTexture(GL_TEXTURE,0);
    glDisable(GL_TEXTURE_2D);
    
    //release the temp image
    [image release];
    
    return [NSDictionary dictionaryWithObjectsAndKeys:
            text,@"text"
            ,[NSNumber numberWithInt:textureName],@"textureName"
            ,[NSNumber numberWithInt:textureWidth],@"textureWidth"
            ,[NSNumber numberWithInt:textureHeight],@"textureHeight"
            , nil];
}

-(void)releaseTextureWithName:(GLuint)textureName andContext:(id<QCPlugInContext>)context
{
    CGLContextObj cgl_ctx = [context CGLContextObj];
    glDeleteTextures(1, &textureName);
}

-(void)releaseAllTexturesWithContext:(id<QCPlugInContext>)context
{
    for(NSDictionary *texture in textures)
    {
        GLuint textureName=(GLuint)[[texture objectForKey:@"textureName"] intValue];
        [self releaseTextureWithName:textureName andContext:context];
    }
    
    [textures removeAllObjects];
}

@end



@implementation MTTextureTypePlugIn(Execution)

- (BOOL) execute:(id<QCPlugInContext>)context atTime:(NSTimeInterval)time withArguments:(NSDictionary*)arguments
{
    CGLContextObj cgl_ctx = [context CGLContextObj];

    if(cgl_ctx==NULL) 
	{
		return NO; //no context, no gain :)
	}
    
    if([self didValueForInputKeyChange:@"inputText"])
    {
        if(self.inputPurge)
        {
            [self releaseAllTexturesWithContext:context];
        }
        
        NSDictionary *attributes=[NSDictionary dictionaryWithObjectsAndKeys:[NSFont fontWithName:@"LucidaGrande" size:200]
                                  ,NSFontAttributeName,[NSColor whiteColor],NSForegroundColorAttributeName
                                  ,[NSColor clearColor],NSBackgroundColorAttributeName
                                  , nil];
        [textures addObject:[self allocTextureWithString:self.inputText attributes:attributes andContext:context]];

        if(nil!=textures && [textures count] > 0)
        {
            NSDictionary *entry=[textures objectAtIndex:0];
            
            self.outputTextureName=[[entry objectForKey:@"textureName"] intValue];
            self.outputTextureWidth=[[entry objectForKey:@"textureWidth"] intValue];
            self.outputTextureHeight=[[entry objectForKey:@"textureHeight"] intValue];
        }
    }
    
    return YES;
}

- (void) stopExecution:(id<QCPlugInContext>)context
{
    [self releaseAllTexturesWithContext:context];
}

@end
